﻿/*
Deployment script for DTLoggedExecDB
*/

GO
SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, CONCAT_NULL_YIELDS_NULL, QUOTED_IDENTIFIER ON;

SET NUMERIC_ROUNDABORT OFF;


GO
:setvar schema_version "18"
:setvar version_date "20110308"

GO
USE [master]

GO
:on error exit
GO
IF (DB_ID(N'$(DatabaseName)') IS NOT NULL
    AND DATABASEPROPERTYEX(N'$(DatabaseName)','Status') <> N'ONLINE')
BEGIN
    RAISERROR(N'The state of the target database, %s, is not set to ONLINE. To deploy to this database, its state must be set to ONLINE.', 16, 127,N'$(DatabaseName)') WITH NOWAIT
    RETURN
END

GO
IF (DB_ID(N'$(DatabaseName)') IS NOT NULL) 
BEGIN
    ALTER DATABASE [$(DatabaseName)]
    SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
    DROP DATABASE [$(DatabaseName)];
END

GO
PRINT N'Creating $(DatabaseName)...'
GO
CREATE DATABASE [$(DatabaseName)]
    ON 
    PRIMARY(NAME = [System], FILENAME = '$(DefaultDataPath)$(DatabaseName).mdf', SIZE = 102400 KB, MAXSIZE = 1048576 KB, FILEGROWTH = 102400 KB), 
    FILEGROUP [SECONDARY](NAME = [UserData], FILENAME = '$(DefaultDataPath)$(DatabaseName)_2.ndf', SIZE = 102400 KB, MAXSIZE = 1048576 KB, FILEGROWTH = 102400 KB), 
    FILEGROUP [XMLDATA](NAME = [XMLDATA], FILENAME = '$(DefaultDataPath)$(DatabaseName)_3.ndf', SIZE = 307200 KB, MAXSIZE = 1048576 KB, FILEGROWTH = 102400 KB)
    LOG ON (NAME = [Log], FILENAME = '$(DefaultLogPath)$(DatabaseName)_log.ldf', SIZE = 153600 KB, MAXSIZE = 1048576 KB, FILEGROWTH = 102400 KB) COLLATE Latin1_General_CI_AS
GO
ALTER DATABASE [$(DatabaseName)]
    MODIFY FILEGROUP [SECONDARY] DEFAULT;


GO
EXECUTE sp_dbcmptlevel [$(DatabaseName)], 90;


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET ANSI_NULLS ON,
                ANSI_PADDING ON,
                ANSI_WARNINGS ON,
                ARITHABORT ON,
                CONCAT_NULL_YIELDS_NULL ON,
                NUMERIC_ROUNDABORT OFF,
                QUOTED_IDENTIFIER ON,
                ANSI_NULL_DEFAULT ON,
                CURSOR_DEFAULT LOCAL,
                RECOVERY FULL,
                CURSOR_CLOSE_ON_COMMIT OFF,
                AUTO_CREATE_STATISTICS ON,
                AUTO_SHRINK OFF,
                AUTO_UPDATE_STATISTICS ON,
                RECURSIVE_TRIGGERS OFF 
            WITH ROLLBACK IMMEDIATE;
        ALTER DATABASE [$(DatabaseName)]
            SET AUTO_CLOSE OFF 
            WITH ROLLBACK IMMEDIATE;
    END


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET ALLOW_SNAPSHOT_ISOLATION OFF;
    END


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET READ_COMMITTED_SNAPSHOT OFF;
    END


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET AUTO_UPDATE_STATISTICS_ASYNC ON,
                PAGE_VERIFY NONE,
                DATE_CORRELATION_OPTIMIZATION OFF,
                DISABLE_BROKER,
                PARAMETERIZATION SIMPLE,
                SUPPLEMENTAL_LOGGING OFF 
            WITH ROLLBACK IMMEDIATE;
    END


GO
IF IS_SRVROLEMEMBER(N'sysadmin') = 1
    BEGIN
        IF EXISTS (SELECT 1
                   FROM   [master].[dbo].[sysdatabases]
                   WHERE  [name] = N'$(DatabaseName)')
            BEGIN
                EXECUTE sp_executesql N'ALTER DATABASE [$(DatabaseName)]
    SET TRUSTWORTHY OFF,
        DB_CHAINING OFF 
    WITH ROLLBACK IMMEDIATE';
            END
    END
ELSE
    BEGIN
        PRINT N'The database settings cannot be modified. You must be a SysAdmin to apply these settings.';
    END


GO
IF IS_SRVROLEMEMBER(N'sysadmin') = 1
    BEGIN
        IF EXISTS (SELECT 1
                   FROM   [master].[dbo].[sysdatabases]
                   WHERE  [name] = N'$(DatabaseName)')
            BEGIN
                EXECUTE sp_executesql N'ALTER DATABASE [$(DatabaseName)]
    SET HONOR_BROKER_PRIORITY OFF 
    WITH ROLLBACK IMMEDIATE';
            END
    END
ELSE
    BEGIN
        PRINT N'The database settings cannot be modified. You must be a SysAdmin to apply these settings.';
    END


GO
USE [$(DatabaseName)]

GO
IF fulltextserviceproperty(N'IsFulltextInstalled') = 1
    EXECUTE sp_fulltext_database 'enable';


GO
/*
 Pre-Deployment Script Template							
--------------------------------------------------------------------------------------
 This file contains SQL statements that will be executed before the build script.	
 Use SQLCMD syntax to include a file in the pre-deployment script.			
 Example:      :r .\myfile.sql								
 Use SQLCMD syntax to reference a variable in the pre-deployment script.		
 Example:      :setvar TableName MyTable							
               SELECT * FROM [$(TableName)]					
--------------------------------------------------------------------------------------
*/

GO
PRINT N'Creating [report]...';


GO
CREATE SCHEMA [report]
    AUTHORIZATION [dbo];


GO
PRINT N'Creating [dbo].[dataflow_profiling]...';


GO
CREATE TABLE [dbo].[dataflow_profiling] (
    [profile_id]           INT      IDENTITY (1, 1) NOT NULL,
    [package_execution_id] INT      NOT NULL,
    [execution_datetime]   DATETIME NOT NULL
);


GO
PRINT N'Creating pk__dataflow_profiling...';


GO
ALTER TABLE [dbo].[dataflow_profiling]
    ADD CONSTRAINT [pk__dataflow_profiling] PRIMARY KEY CLUSTERED ([profile_id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creating uk__dataflow_profiling_1...';


GO
ALTER TABLE [dbo].[dataflow_profiling]
    ADD CONSTRAINT [uk__dataflow_profiling_1] UNIQUE NONCLUSTERED ([package_execution_id] ASC, [execution_datetime] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [SECONDARY];


GO
PRINT N'Creating [dbo].[dataflow_profiling_buffers_data]...';


GO
CREATE TABLE [dbo].[dataflow_profiling_buffers_data] (
    [profile_id]       INT              NOT NULL,
    [dataflow_id]      UNIQUEIDENTIFIER NOT NULL,
    [dataflow_name]    VARCHAR (100)    NOT NULL,
    [execution_count]  INT              NOT NULL,
    [source_id]        INT              NOT NULL,
    [source_name]      VARCHAR (100)    NOT NULL,
    [component_id]     INT              NOT NULL,
    [component_name]   VARCHAR (100)    NOT NULL,
    [destination_id]   INT              NOT NULL,
    [destination_name] VARCHAR (100)    NOT NULL,
    [row_count]        BIGINT           NOT NULL,
    [event_datetime]   DATETIME         NOT NULL
) WITH (DATA_COMPRESSION = PAGE);


GO
PRINT N'Creating [dbo].[dataflow_profiling_buffers_data].[ixc__dataflow_profiling_data_1]...';


GO
CREATE CLUSTERED INDEX [ixc__dataflow_profiling_data_1]
    ON [dbo].[dataflow_profiling_buffers_data]([profile_id] ASC, [event_datetime] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF, ONLINE = OFF, MAXDOP = 0);


GO
PRINT N'Creating [dbo].[dataflow_profiling_components_data]...';


GO
CREATE TABLE [dbo].[dataflow_profiling_components_data] (
    [profile_id]                INT              NOT NULL,
    [dataflow_id]               UNIQUEIDENTIFIER NOT NULL,
    [dataflow_name]             VARCHAR (100)    NOT NULL,
    [execution_count]           INT              NOT NULL,
    [source_id]                 INT              NOT NULL,
    [source_name]               VARCHAR (100)    NOT NULL,
    [component_id]              INT              NOT NULL,
    [component_name]            VARCHAR (100)    NOT NULL,
    [destination_id]            INT              NOT NULL,
    [destination_name]          VARCHAR (100)    NOT NULL,
    [execution_start]           DATETIME         NOT NULL,
    [execution_end]             DATETIME         NOT NULL,
    [execution_elapsed_seconds] BIGINT           NOT NULL,
    [processed_rows]            BIGINT           NOT NULL
) WITH (DATA_COMPRESSION = PAGE);


GO
PRINT N'Creating [dbo].[dataflow_profiling_components_data].[ixc__dataflow_profiling_components_data_1]...';


GO
CREATE CLUSTERED INDEX [ixc__dataflow_profiling_components_data_1]
    ON [dbo].[dataflow_profiling_components_data]([profile_id] ASC, [execution_start] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF, ONLINE = OFF, MAXDOP = 0);


GO
PRINT N'Creating [dbo].[dataflow_profiling_dataflows_data]...';


GO
CREATE TABLE [dbo].[dataflow_profiling_dataflows_data] (
    [profile_id]                INT              NOT NULL,
    [dataflow_id]               UNIQUEIDENTIFIER NOT NULL,
    [dataflow_name]             VARCHAR (100)    NOT NULL,
    [execution_count]           INT              NOT NULL,
    [execution_start]           DATETIME         NOT NULL,
    [execution_end]             DATETIME         NOT NULL,
    [execution_elapsed_seconds] BIGINT           NOT NULL,
    [processed_rows]            BIGINT           NOT NULL
) WITH (DATA_COMPRESSION = PAGE);


GO
PRINT N'Creating [dbo].[dataflow_profiling_dataflows_data].[ixc__dataflow_profiling_dataflows_data_1]...';


GO
CREATE CLUSTERED INDEX [ixc__dataflow_profiling_dataflows_data_1]
    ON [dbo].[dataflow_profiling_dataflows_data]([profile_id] ASC, [execution_start] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF, ONLINE = OFF, MAXDOP = 0);


GO
PRINT N'Creating [dbo].[package_execution_logs]...';


GO
CREATE TABLE [dbo].[package_execution_logs] (
    [row_id]               INT              IDENTITY (1, 1) NOT NULL,
    [package_execution_id] INT              NOT NULL,
    [event_type]           VARCHAR (100)    NOT NULL,
    [task_name]            VARCHAR (100)    NULL,
    [package_name]         VARCHAR (100)    NULL,
    [task_type]            VARCHAR (1024)   NULL,
    [description]          VARCHAR (100)    NULL,
    [object_id]            UNIQUEIDENTIFIER NULL,
    [container_name]       VARCHAR (100)    NULL,
    [container_id]         UNIQUEIDENTIFIER NULL,
    [start_datetime]       DATETIME         NULL,
    [end_datetime]         DATETIME         NULL,
    [message]              VARCHAR (MAX)    NULL,
    [variables]            XML              NULL,
    [expressions]          XML              NULL,
    [properties]           XML              NULL
) TEXTIMAGE_ON [XMLDATA] WITH (DATA_COMPRESSION = PAGE);


GO
PRINT N'Creating [dbo].[package_execution_logs].[ixc__package_execution_logs_1]...';


GO
CREATE CLUSTERED INDEX [ixc__package_execution_logs_1]
    ON [dbo].[package_execution_logs]([package_execution_id] ASC, [start_datetime] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF, ONLINE = OFF, MAXDOP = 0);


GO
PRINT N'Creating [dbo].[package_execution_results]...';


GO
CREATE TABLE [dbo].[package_execution_results] (
    [row_id]                 INT          IDENTITY (1, 1) NOT NULL,
    [package_execution_id]   INT          NOT NULL,
    [package_start_datetime] DATETIME     NULL,
    [package_end_datetime]   DATETIME     NULL,
    [load_start_datetime]    DATETIME     NULL,
    [load_end_datetime]      DATETIME     NULL,
    [package_result]         VARCHAR (50) NULL,
    [load_result]            VARCHAR (50) NULL,
    [warnings]               INT          NULL,
    [errors]                 INT          NULL
);


GO
PRINT N'Creating [dbo].[package_execution_results].[ixc__package_execution_results_1]...';


GO
CREATE CLUSTERED INDEX [ixc__package_execution_results_1]
    ON [dbo].[package_execution_results]([package_execution_id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF, ONLINE = OFF, MAXDOP = 0);


GO
PRINT N'Creating [dbo].[package_executions]...';


GO
CREATE TABLE [dbo].[package_executions] (
    [package_execution_id] INT              IDENTITY (1, 1) NOT NULL,
    [package_id]           INT              NOT NULL,
    [user_name]            NVARCHAR (128)   NOT NULL,
    [machine_name]         NVARCHAR (128)   NOT NULL,
    [execution_datetime]   DATETIME         NOT NULL,
    [execution_guid]       UNIQUEIDENTIFIER NOT NULL
);


GO
PRINT N'Creating pk__package_executions...';


GO
ALTER TABLE [dbo].[package_executions]
    ADD CONSTRAINT [pk__package_executions] PRIMARY KEY CLUSTERED ([package_execution_id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creating uk__package_executions_1...';


GO
ALTER TABLE [dbo].[package_executions]
    ADD CONSTRAINT [uk__package_executions_1] UNIQUE NONCLUSTERED ([package_id] ASC, [execution_guid] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [SECONDARY];


GO
PRINT N'Creating [dbo].[packages]...';


GO
CREATE TABLE [dbo].[packages] (
    [package_id]   INT              IDENTITY (1, 1) NOT NULL,
    [name]         VARCHAR (512)    NOT NULL,
    [version]      VARCHAR (15)     NOT NULL,
    [package_guid] UNIQUEIDENTIFIER NOT NULL,
    [version_guid] UNIQUEIDENTIFIER NOT NULL,
    [project_id]   INT              NOT NULL,
    [section_id]   INT              NOT NULL
);


GO
PRINT N'Creating pk__packages...';


GO
ALTER TABLE [dbo].[packages]
    ADD CONSTRAINT [pk__packages] PRIMARY KEY CLUSTERED ([package_id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creating uk__packages_1...';


GO
ALTER TABLE [dbo].[packages]
    ADD CONSTRAINT [uk__packages_1] UNIQUE NONCLUSTERED ([package_guid] ASC, [version_guid] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [SECONDARY];


GO
PRINT N'Creating [dbo].[projects]...';


GO
CREATE TABLE [dbo].[projects] (
    [project_id] INT           IDENTITY (1, 1) NOT NULL,
    [name]       VARCHAR (512) NOT NULL,
    PRIMARY KEY CLUSTERED ([project_id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF)
) ON [PRIMARY];


GO
PRINT N'Creating uc__projects__1...';


GO
ALTER TABLE [dbo].[projects]
    ADD CONSTRAINT [uc__projects__1] UNIQUE NONCLUSTERED ([name] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [PRIMARY];


GO
PRINT N'Creating [dbo].[sections]...';


GO
CREATE TABLE [dbo].[sections] (
    [section_id] INT           IDENTITY (1, 1) NOT NULL,
    [name]       VARCHAR (512) NOT NULL
);


GO
PRINT N'Creating PK__section__F842676A5165187F...';


GO
ALTER TABLE [dbo].[sections]
    ADD CONSTRAINT [PK__section__F842676A5165187F] PRIMARY KEY CLUSTERED ([section_id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creating [dbo].[version]...';


GO
CREATE TABLE [dbo].[version] (
    [schema_version] INT           NOT NULL,
    [version_date]   SMALLDATETIME NOT NULL
);


GO
PRINT N'Creating pk__version...';


GO
ALTER TABLE [dbo].[version]
    ADD CONSTRAINT [pk__version] PRIMARY KEY CLUSTERED ([schema_version] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creating On column: project_id...';


GO
ALTER TABLE [dbo].[packages]
    ADD DEFAULT ((-1)) FOR [project_id];


GO
PRINT N'Creating DF__packages__sectio__534D60F1...';


GO
ALTER TABLE [dbo].[packages]
    ADD CONSTRAINT [DF__packages__sectio__534D60F1] DEFAULT ((-1)) FOR [section_id];


GO
PRINT N'Creating fk__dataflow_profiling__package_executions...';


GO
ALTER TABLE [dbo].[dataflow_profiling] WITH NOCHECK
    ADD CONSTRAINT [fk__dataflow_profiling__package_executions] FOREIGN KEY ([package_execution_id]) REFERENCES [dbo].[package_executions] ([package_execution_id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creating fk__dataflow_profiling_data__dataflow_profiling...';


GO
ALTER TABLE [dbo].[dataflow_profiling_buffers_data] WITH NOCHECK
    ADD CONSTRAINT [fk__dataflow_profiling_data__dataflow_profiling] FOREIGN KEY ([profile_id]) REFERENCES [dbo].[dataflow_profiling] ([profile_id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creating fk__package_execution_logs__package_executions...';


GO
ALTER TABLE [dbo].[package_execution_logs] WITH NOCHECK
    ADD CONSTRAINT [fk__package_execution_logs__package_executions] FOREIGN KEY ([package_execution_id]) REFERENCES [dbo].[package_executions] ([package_execution_id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creating fk__package_execution_results__package_executions...';


GO
ALTER TABLE [dbo].[package_execution_results] WITH NOCHECK
    ADD CONSTRAINT [fk__package_execution_results__package_executions] FOREIGN KEY ([package_execution_id]) REFERENCES [dbo].[package_executions] ([package_execution_id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creating fk__package_executions__packages...';


GO
ALTER TABLE [dbo].[package_executions] WITH NOCHECK
    ADD CONSTRAINT [fk__package_executions__packages] FOREIGN KEY ([package_id]) REFERENCES [dbo].[packages] ([package_id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creating fk__packages__projects...';


GO
ALTER TABLE [dbo].[packages] WITH NOCHECK
    ADD CONSTRAINT [fk__packages__projects] FOREIGN KEY ([project_id]) REFERENCES [dbo].[projects] ([project_id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creating fk__packages__sections...';


GO
ALTER TABLE [dbo].[packages] WITH NOCHECK
    ADD CONSTRAINT [fk__packages__sections] FOREIGN KEY ([section_id]) REFERENCES [dbo].[sections] ([section_id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creating [report].[stp_get_package_execution_data]...';


GO
CREATE PROCEDURE [report].[stp_get_package_execution_data]
@package_execution_id INT, @level INT=1
AS
with cte_base_pre_post_event_data as
(
select
	rn = ROW_NUMBER() over (partition by pre.row_id order by post.row_id),
	pre_row_id = pre.row_id,
	--pre_start_datetime = pre.start_datetime,
	--pre_end_datetime = pre.end_datetime,
	post.row_id,
	post.package_execution_id, 	
	post.package_name,
	post.task_name, 
	post.description, 
	post.object_id, 
	post.container_name, 
	post.container_id, 
	post.start_datetime, 
	post.end_datetime
from
	dbo.package_execution_logs pre
inner join
	dbo.package_execution_logs post on	pre.object_id = post.object_id and 
										pre.event_type = 'OnPreExecute' and 
										post.event_type = 'OnPostExecute' and 
										pre.package_execution_id = post.package_execution_id and
										post.row_id > pre.row_id
where
	pre.container_id <> pre.object_id
and
	post.container_id <> post.object_id
and
	pre.package_execution_id = @package_execution_id
),
cte_pre_post_event_data as 
(
select
	*,
	elapsed_time_secs = DATEDIFF(ss, start_datetime, end_datetime)
from
	cte_base_pre_post_event_data
where
	rn = 1	
),
cte_package_events as
(
select
	row_id = min(pre_row_id), package_execution_id, package_name, task_name, description, object_id, container_name, container_id, start_datetime = min(start_datetime), end_datetime = max(end_datetime),
	execution_count = COUNT(*), avg_elapsed_time_secs = AVG(elapsed_time_secs)
from
	cte_pre_post_event_data pel
where
	pel.package_execution_id = @package_execution_id and pel.container_id <> pel.object_id
group by
	package_execution_id, package_name, task_name, description, object_id, container_name, container_id
)
select
	*
into
	#t
from
	cte_package_events


;with cte_package_start as
(
	select			
		--pre_start_datetime = pre.start_datetime,
		--pre_end_datetime = pre.end_datetime,
		pre.row_id,
		post.package_execution_id, 	
		post.package_name,
		post.task_name, 
		post.description, 
		post.object_id, 
		post.container_name, 
		post.container_id, 
		pre.start_datetime, 
		post.end_datetime,
		execution_count = 1,
		avg_elapsed_time_secs = DATEDIFF(ss, pre.start_datetime, post.end_datetime)
	from
		dbo.package_execution_logs pre
	left join
		dbo.package_execution_logs post on	pre.object_id = post.object_id and 
											pre.event_type = 'PackageStart' and 
											post.event_type = 'PackageEnd' and 
											pre.package_execution_id = post.package_execution_id and
											post.row_id > pre.row_id
	where	
		pre.package_execution_id = @package_execution_id
	and
		pre.event_type = 'PackageStart'	
),
cte_hiearchy as
(
	select
		[level] = 0,
		[ordered_path] = cast('/' as varchar(max)),
		[name_path] = cast('/' as varchar(max)),
		*
	from
		cte_package_start
		
	union all
	
	select		
		[level] = [level] + 1,		
		[ordered_path] = cast([ordered_path] + cast(c.row_id as varchar(9)) + '/' as varchar(max)),		
		[name_path] = cast([name_path] + c.[task_name] + '/' as varchar(max)),
		c.*
	from
		#t c
	inner join 
		cte_hiearchy p on c.container_id = p.object_id 
)
select
	[ordered_path],
	c.row_id,
	[name_path],
	c.package_name,
	c.level,
	task_name = ISNULL(c.task_name, 'Package'),
	c.object_id,
	c.description, 	
	c.start_datetime, 
	c.end_datetime,
	elapsed_seconds = DATEDIFF(ss, c.start_datetime, c.end_datetime),
	elapsed_seconds_from_start = DATEDIFF(ss, s.start_datetime, c.start_datetime),
	c.execution_count,
	avg_iteration_time_secs = c.avg_elapsed_time_secs,
	total_iteration_time_secs = c.avg_elapsed_time_secs * c.execution_count
from
	cte_hiearchy c
cross join
	cte_package_start s	
where
	c.end_datetime >= c.start_datetime
and
	level <= @level
order by
	[ordered_path]
GO
PRINT N'Creating [dbo].[vw_dataflow_profiled]...';


GO
CREATE VIEW [dbo].[vw_dataflow_profiled]
AS
select
	p.[package_id],
	p.[name],
	p.[version],
	pe.[package_execution_id],
	package_execution_datetime = pe.[execution_datetime],
	dp.[profile_id],
	dataflow_execution_datetime = dp.[execution_datetime]
from	
	dbo.[dataflow_profiling] as dp
inner join
	dbo.[package_executions] as pe on dp.package_execution_id = pe.package_execution_id
inner join
	dbo.[packages] as p on pe.[package_id] = p.[package_id];
GO
PRINT N'Creating [dbo].[vw_dataflow_profiled_buffers_data]...';


GO
CREATE VIEW [dbo].[vw_dataflow_profiled_buffers_data]
AS
select
	p.[package_id],
	p.[package_guid],
	package_version_guid = p.[version_guid],
	p.[name],
	p.[version],
	pe.[package_execution_id],
	package_execution_datetime = pe.[execution_datetime],
	dp.[profile_id],
	dataflow_execution_datetime = dp.[execution_datetime],
	[dpbd].[dataflow_id], 
	[dpbd].[dataflow_name], 
	[dpbd].[execution_count], 
	[dpbd].[source_id], 
	[dpbd].[source_name], 
	[dpbd].[component_id], 
	[dpbd].[component_name], 
	[dpbd].[destination_id], 
	[dpbd].[destination_name], 
	[dpbd].[row_count], 
	[dpbd].[event_datetime]
from
	dbo.[dataflow_profiling_buffers_data] as dpbd 
inner join
	dbo.[dataflow_profiling] as dp on dp.[profile_id] = dpbd.[profile_id]
inner join
	dbo.[package_executions] as pe on dp.package_execution_id = pe.package_execution_id
inner join
	dbo.[packages] as p on pe.[package_id] = p.[package_id];
GO
PRINT N'Creating [dbo].[vw_dataflow_profiled_components_data]...';


GO
CREATE VIEW [dbo].[vw_dataflow_profiled_components_data]
AS
SELECT
	p.package_guid,
	package_version_guid = p.version_guid,
	p.[package_id],
	p.name AS package_name,
	p.version AS package_version,
	pe.[package_execution_id],
	package_execution_datetime = pe.[execution_datetime],
	dp.[profile_id],
	dataflow_execution_datetime = dp.[execution_datetime],
	[dpcd].[dataflow_id], 
	[dpcd].[dataflow_name], 
	[dpcd].[execution_count], 
	[dpcd].[source_id], 
	[dpcd].[source_name], 
	[dpcd].[component_id], 
	[dpcd].[component_name], 
	[dpcd].[destination_id], 
	[dpcd].[destination_name], 
	[dpcd].[execution_start], 
	[dpcd].[execution_end], 
	[dpcd].[execution_elapsed_seconds], 
	[dpcd].[processed_rows]
from
	dbo.[dataflow_profiling_components_data] as dpcd 
inner join
	dbo.[dataflow_profiling] as dp on dp.[profile_id] = dpcd.[profile_id]
inner join
	dbo.[package_executions] as pe on dp.package_execution_id = pe.package_execution_id
inner join
	dbo.[packages] as p on pe.[package_id] = p.[package_id];
GO
PRINT N'Creating [dbo].[vw_dataflow_profiled_components_statistics]...';


GO
CREATE VIEW [dbo].[vw_dataflow_profiled_components_statistics]
AS
SELECT 
	dp.package_execution_id,
	dpcd.profile_id,
	dpcd.dataflow_id,
	components = COUNT(dpcd.profile_id),
	processed_rows = SUM(dpcd.processed_rows),
	avg_processed_rows_per_component = (1. * SUM(dpcd.processed_rows)) / COUNT(dpcd.profile_id) 
FROM 
	[dbo].[dataflow_profiling_components_data] dpcd
INNER JOIN
	[dbo].[dataflow_profiling] dp ON dpcd.profile_id = dp.profile_id
GROUP BY 
	dp.package_execution_id,
	dpcd.dataflow_id,
	dpcd.profile_id;
GO
PRINT N'Creating [dbo].[vw_dataflow_profiled_dataflows_data]...';


GO
CREATE VIEW [dbo].[vw_dataflow_profiled_dataflows_data]
AS
SELECT
	p.package_guid,
	package_version_guid = p.version_guid,
	p.[package_id],
	p.[name],
	p.[version],
	pe.[package_execution_id],
	package_execution_datetime = pe.[execution_datetime],
	dp.[profile_id],
	dataflow_execution_datetime = dp.[execution_datetime],
	dpdd.[dataflow_id],
	dpdd.[dataflow_name],
	dpdd.[execution_count],
	dpdd.[execution_start],
	dpdd.[execution_end],
	dpdd.[execution_elapsed_seconds],
	dpdd.[processed_rows]
from
	dbo.[dataflow_profiling_dataflows_data] as dpdd 
inner join
	dbo.[dataflow_profiling] as dp on dp.[profile_id] = dpdd.[profile_id]
inner join
	dbo.[package_executions] as pe on dp.package_execution_id = pe.package_execution_id
inner join
	dbo.[packages] as p on pe.[package_id] = p.[package_id];
GO
PRINT N'Creating [dbo].[vw_executed_packages]...';


GO


CREATE view [dbo].[vw_executed_packages]
as
select 
	p.package_id,
	pe.package_execution_id,
	project_name = prj.name,
	section_name = s.name,
	package_name = p.name,
	package_version = p.version,
	package_guid,
	version_guid,
	execution_owner = user_name,
	execution_server = machine_name,
	execution_datetime,
	load_start_datetime,
	load_end_datetime,
	load_elapsed_time_secs = DATEDIFF(ss, load_start_datetime, load_end_datetime),
	package_start_datetime,
	package_end_datetime,
	package_elapsed_time_secs = DATEDIFF(ss, package_start_datetime, package_end_datetime),
	load_result,
	package_result,
	warnings,
	errors
from
	dbo.packages p
inner join
	dbo.package_executions pe on p.package_id = pe.package_id
inner join
	dbo.package_execution_results per on per.package_execution_id = pe.package_execution_id
inner join
	dbo.projects prj on p.project_id = prj.project_id 
inner join
	dbo.sections s on p.[section_id] = s.[section_id]
GO
-- Refactoring step to update target server with deployed transaction logs
CREATE TABLE  [dbo].[__RefactorLog] (OperationKey UNIQUEIDENTIFIER NOT NULL PRIMARY KEY)
GO
sp_addextendedproperty N'microsoft_database_tools_support', N'refactoring log', N'schema', N'dbo', N'table', N'__RefactorLog'
GO

GO
/*
Post-Deployment Script Template							
--------------------------------------------------------------------------------------
 This file contains SQL statements that will be appended to the build script.		
 Use SQLCMD syntax to include a file in the post-deployment script.			
 Example:      :r .\myfile.sql								
 Use SQLCMD syntax to reference a variable in the post-deployment script.		
 Example:      :setvar TableName MyTable							
               SELECT * FROM [$(TableName)]					
--------------------------------------------------------------------------------------
*/

:on error exit

insert into dbo.[version] values ($(schema_version), '$(version_date)')

set identity_insert dbo.[projects] on
insert into dbo.[projects] (project_id, [name]) values (-1, 'Undefined')
set identity_insert dbo.[projects] off

set identity_insert dbo.[sections] on
insert into dbo.[sections] (section_id, [name]) values (-1, 'Undefined')
set identity_insert dbo.[sections] off


GO
PRINT N'Checking existing data against newly created constraints';


GO
USE [$(DatabaseName)];


GO
ALTER TABLE [dbo].[dataflow_profiling] WITH CHECK CHECK CONSTRAINT [fk__dataflow_profiling__package_executions];

ALTER TABLE [dbo].[dataflow_profiling_buffers_data] WITH CHECK CHECK CONSTRAINT [fk__dataflow_profiling_data__dataflow_profiling];

ALTER TABLE [dbo].[package_execution_logs] WITH CHECK CHECK CONSTRAINT [fk__package_execution_logs__package_executions];

ALTER TABLE [dbo].[package_execution_results] WITH CHECK CHECK CONSTRAINT [fk__package_execution_results__package_executions];

ALTER TABLE [dbo].[package_executions] WITH CHECK CHECK CONSTRAINT [fk__package_executions__packages];

ALTER TABLE [dbo].[packages] WITH CHECK CHECK CONSTRAINT [fk__packages__projects];

ALTER TABLE [dbo].[packages] WITH CHECK CHECK CONSTRAINT [fk__packages__sections];


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        DECLARE @VarDecimalSupported AS BIT;
        SELECT @VarDecimalSupported = 0;
        IF ((ServerProperty(N'EngineEdition') = 3)
            AND (((@@microsoftversion / power(2, 24) = 9)
                  AND (@@microsoftversion & 0xffff >= 3024))
                 OR ((@@microsoftversion / power(2, 24) = 10)
                     AND (@@microsoftversion & 0xffff >= 1600))))
            SELECT @VarDecimalSupported = 1;
        IF (@VarDecimalSupported > 0)
            BEGIN
                EXECUTE sp_db_vardecimal_storage_format N'$(DatabaseName)', 'ON';
            END
    END


GO
