------------------------------------------------------------------------
-- Script:			import-log-data.sql
-- Author:			Davide Mauri (SolidQ)
-- Version:         18
-- Credits:			-
-- Copyright:		Attribution-NonCommercial-ShareAlike 2.5
-- Target Version:	SQL Server 2005 or above
-- Tab/indent size:	4
-- Notes:			-
------------------------------------------------------------------------
set nocount on
set quoted_identifier on
go

print 'Import Log Data Batch v. 18'
go

:setvar required_db_version 19

IF (LEN('$(project_name)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "project_name" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(section_name)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "section_name" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(dtloggedexec_db_name)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "dtloggedexec_db_name" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(log_path)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "log_path" must be specified.',16,127) WITH NOWAIT
END
GO

IF (LEN('$(log_file)') = 0) 
BEGIN
	RAISERROR('SQLCMD Variable "log_file" must be specified.',16,127) WITH NOWAIT
END
GO

use [$(dtloggedexec_db_name)]
go

print 'Logs are bound to project "$(project_name)", section "$(section_name)"'
print 'Reading file "$(log_path)$(log_file)"'
print 'Using format file "$(log_path)formatheader.fmt"'
go

/*

	check for compatibility

*/
--Check that db has been updated to the correct schema version
declare @dbschemaversion int;
select top 1 @dbschemaversion = schema_version from dbo.[version] order by version_date desc;

if (@dbschemaversion < $(required_db_version))
begin
	raiserror ('DTLoggedExecDB must be updated to schema version  $(required_db_version) or later.', 16, 127) with nowait
end
go

--Get File Type Version
declare @fileVersion as int;
declare @readFileVersion as varchar(100)
with cte as
(
select 
	[field] = substring(Header, charindex('#', Header) + 1, charindex(': ', Header) - charindex('#', Header) - 1),
	[value] = substring(Header, charindex(': ', Header) + 2, 512)
from 
	openrowset
	(
		bulk '$(log_path)$(log_file)', 
		formatfile='$(log_path)formatheader.fmt', 
		firstrow = 0, lastrow = 1
	) t
)
select
	@readFileVersion = [value]
from
	cte
where
	[field] = '__DataFileVersion'
;

begin try
	set @fileVersion = isnull(cast(@readFileVersion as int), 1)
end try
begin catch
	set @fileVersion = 1
end catch

print 'Found log file version: ' + cast(@fileVersion as varchar(3));
if (@fileVersion < 3) 
begin
	raiserror ('Loaded file must have a version 3 header at least to continue.', 16, 127) with nowait
end

declare @lr varchar(2);
set @lr = null;	
if (@fileVersion = 3) set @lr = '9';
if (@fileVersion = 4) set @lr = '11';
	
if (@lr is null) 
begin
	raiserror ('Unexpected file version.', 16, 127) with nowait
end

	
/*

	Read Header Data

*/
print 'Loading header data...'
if (object_id('tempdb..#header') is not null) drop table #header;

create table #header 
(
	[file_status] [varchar](100) NULL,
	[package] [varchar](100) NULL,
	[version] [varchar](100) NULL,
	[package_guid] [uniqueidentifier] NULL,
	[version_guid] [uniqueidentifier] NULL,
	[execution_guid] [uniqueidentifier] NULL,
	[user_name] [varchar](100) NULL,
	[machine_name] [varchar](100) NULL,
	[execution_datetime] [datetime] NULL,
	[execution_label] [varchar](100) NULL
); 
insert into
	#header 
exec('
	with cte as
	(
		select 
			[field] = substring(Header, charindex(''#'', Header) + 1, charindex('': '', Header) - charindex(''#'', Header) - 1),
			[value] = substring(Header, charindex('': '', Header) + 2, 512)
		from 
			openrowset
			(
				bulk ''$(log_path)$(log_file)'', 
				formatfile=''$(log_path)formatheader.fmt'', 
				firstrow = 2, lastrow = ' + @lr + '
			) t
	)
	select
		[file_status] = FileStatus,	
		[package],
		[version],
		[package_guid] = cast(substring(PackageGUID, 2, 36) as uniqueidentifier),
		[version_guid] = cast(substring(VersionGUID, 2, 36) as uniqueidentifier),
		[execution_guid] = cast(substring(ExecutionGUID, 2, 36) as uniqueidentifier),
		[user_name] = UserName,
		[machine_name] = MachineName,
		[execution_datetime] = cast(ExecutionDateTime as datetime),
		[execution_label] = ExecutionLabel


	from
		cte
	pivot
		(
			max(value) for [field] in (FileStatus, Package, Version, PackageGUID, VersionGUID, ExecutionGUID, UserName, MachineName, ExecutionDateTime, ExecutionLabel)
		) p
');

declare @label varchar(512);

if (@fileVersion = 4) begin
	declare @file_status varchar(100);
	select top 1 @file_status = file_status from #header;
	print 'File status = ' +  @file_status;

	if (@file_status != 'CLOSE') begin
		raiserror ('Cannot read log. File wasn''t closed correctly.', 16, 127) with nowait
	end

	select top 1 @label = execution_label from #header;
	set @label = NULLIF(RTRIM(LTRIM(@label)), '')
end


/*
	Save project data into dbo.projects table, or load project_id value
	if project already exists
*/
declare @project_id int;
set @project_id = (select project_id from [dbo].[projects] p where p.name = '$(project_name)')

if (@project_id is null) begin
	insert into dbo.[projects] ([name]) values ('$(project_name)');
	
	set @project_id = scope_identity()
end

/*
	Save section data into dbo.section, or load project_id value
	if project already exists
*/
declare @section_id int;
set @section_id = (select section_id from [dbo].[sections] s where s.name = '$(section_name)')

if (@section_id is null) begin
	insert into dbo.[sections] ([name]) values ('$(section_name)');
	
	set @section_id = scope_identity()
end

/*
	Save label data into dbo.labels table, or load label_id value
	if label already exists
*/
declare @label_id int;
if (@label is null) begin
	set @label_id = -1;
end else begin
	set @label_id = (select label_id from [dbo].[labels] l where l.label = @label)

	if (@label_id is null) begin
		insert into dbo.[labels] (label) values (@label);
		
		set @label_id = scope_identity()
	end
end

/*

	Save package data into dbo.packages table, or load package_id value 
	if package data already exists

*/
declare @package_id int;

set @package_id = (select package_id from [dbo].[packages] p inner join #header h on p.package_guid = h.package_guid and p.version_guid = h.version_guid)

if (@package_id is null) begin
	insert into dbo.[packages] 
	select top 1 [package], [version], [package_guid], [version_guid], @project_id, @section_id from #header;
	
	set @package_id = scope_identity()
end else begin
	update dbo.[packages] set project_id = @project_id, section_id = @section_id where package_id = @package_id
end


/*

	Save package execution data into dbo.package_executions

*/
declare @hasError int
set @hasError = 0
begin try
	begin tran

	declare @execution_id int;

	set @execution_id = (select package_execution_id from [dbo].[package_executions] p inner join #header h on p.execution_guid = h.execution_guid and p.package_id = @package_id)

	if (@execution_id is not null) begin
		if (exists(select top 1 * from [dbo].[package_execution_logs] where package_execution_id = @execution_id)) begin
			drop table #header
			raiserror ('Package Log already loaded', 16, 127) with nowait			
		end
	end

	if (@execution_id is null) begin
		insert into dbo.[package_executions] ([package_id], [user_name], [machine_name], [execution_datetime], [execution_guid], [label_id]) 
		select top 1 @package_id, [user_name], [machine_name], [execution_datetime], [execution_guid], @label_id from [#header] as h

		set @execution_id = scope_identity()
	end


	/*

		Save package log into dbo.package_execution_logs

	*/
	print 'Loading logs...'

	insert into
		[dbo].[package_execution_logs]
	select
		@execution_id,
		event_type =		EventType,
		task_name =			TaskName,
		package_name =		PackageName,
		task_type =			TaskType,	
		[description] =		[Description],
		[object_id] =		cast(substring(ObjectId, 2, 36) as uniqueidentifier),
		containter_name =	ContainerName,
		container_id =		cast(substring(ContainerId, 2, 36) as uniqueidentifier),
		start_datetime =	cast(StartTime as datetime),
		end_datetime =		cast(EndTime as datetime),
		[message] =			[Message],
		variables =			cast(Variables as xml),
		expressions =		cast(Expressions as xml),
		properties =		cast(Properties as xml)
	from
		openrowset
		(
			bulk '$(log_path)$(log_file)', 
			formatfile='$(log_path)csvlog.fmt', 
			firstrow = 2
		) t;
	
	
	PRINT 'Extracting package load and execution data...';
	
	WITH cte AS 
	(
		SELECT 
			package_execution_id,
			event_type,
			event_datetime =
							CASE 
								WHEN event_type IN ('PackageEnd', 'PackageLoadEnd') THEN end_datetime
								ELSE start_datetime		
							END,	
			result = 
					CASE 
						WHEN event_type IN ('PackageEnd', 'PackageLoadEnd') THEN CAST([message] AS XML).value('(//result)[1]', 'varchar(50)')
						ELSE NULL
					END,
			errors = 
					CASE 
						WHEN event_type IN ('PackageEnd') THEN CAST([message] AS XML).value('(//errors)[1]', 'int')
						ELSE NULL
					END, 			 
			warnings = 
					CASE 
						WHEN event_type IN ('PackageEnd') THEN CAST([message] AS XML).value('(//warnings)[1]', 'int')
						ELSE NULL
					END  
		FROM 
			dbo.package_execution_logs pel 
		WHERE 
			package_execution_id = @execution_id
		AND
			event_type IN ('PackageStart', 'PackageEnd', 'PackageLoadStart', 'PackageLoadEnd')	
	),
	cte_time AS
	(
		SELECT
			package_execution_id,
			event_type,
			event_datetime 
		FROM
			cte
	),
	cte_result AS
	(
		SELECT
			package_execution_id,
			event_type,
			result
		FROM
			cte
	),
	cte_stats AS
	(
		SELECT
			package_execution_id,			
			warnings,
			errors
		FROM
			cte
		WHERE
			event_type = 'PackageEnd'
	),
	cte_time_pivoted AS 
	(
		SELECT
			package_execution_id,
			package_start_datetime = PackageStart, 
			package_end_datetime = PackageEnd, 
			load_start_datetime = PackageLoadStart, 
			load_end_datetime = PackageLoadEnd
		FROM
			cte_time
		PIVOT
			(
				MAX(event_datetime) FOR [event_type] IN (PackageStart, PackageEnd, PackageLoadStart, PackageLoadEnd)
			) p
	),
	cte_result_pivoted AS 
	(
		SELECT
			package_execution_id,		
			package_result = PackageEnd, 
			load_result = PackageLoadEnd		
		FROM
			cte_result
		PIVOT
			(
				MAX(result) FOR [event_type] IN (PackageStart, PackageEnd, PackageLoadStart, PackageLoadEnd)				
			) p
	)
	INSERT INTO
		dbo.package_execution_results
	SELECT
		t.package_execution_id,
		t.package_start_datetime, 
		t.package_end_datetime, 
		t.load_start_datetime, 
		t.load_end_datetime,
		r.package_result,
		r.load_result,		
		s.warnings,
		s.errors
	FROM
		cte_result_pivoted r 
	INNER JOIN
		cte_time_pivoted t ON r.package_execution_id = t.package_execution_id
	LEFT OUTER JOIN
		cte_stats s ON r.package_execution_id = s.package_execution_id
	
	
	commit tran
end try
begin catch
	print 'ERROR FOUND: ' + error_message();
	if (@@trancount > 0) rollback tran;	
	set @hasError = 1
end catch		
	
/*

	Cleanup
	
*/

print 'Cleaning Up...'

drop table #header;

if (@hasError = 0) 
begin
	print 'Done'
end else begin
	raiserror ('Batch Executed With Errors', 16, 127)  with nowait
end
