﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using System.Xml;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;

namespace DTLoggedExec.LogProviders
{    
    public class CSVLogProvider : ILogProvider, IDisposable
    {
        private class PackageInfo
        {
            public string Name { get; set; }
            public string Version { get; set; }
            public string PackageGUID { get; set; }
            public string VersionGUID { get; set; }
            public string ExecutionGUID { get; set; }
            public string UserName { get; set; }
            public string MachineName { get; set; }
            public string ExecutionDateTime { get; set; }
            public string ExecutionLabel { get; set; }

            public PackageInfo()
            {
                Name = string.Empty;
                Version = string.Empty;
                PackageGUID = string.Empty;
                VersionGUID = string.Empty;
                ExecutionGUID = string.Empty;
                UserName = string.Empty;
                MachineName = string.Empty;
                ExecutionDateTime = string.Empty;
                ExecutionLabel = string.Empty;
            }
        }

        private class EventData
        {
            public string EventType { get; set; }
            public string TaskName { get; set; }
            public string PackageName { get; set; }
            public string TaskType { get; set; }
            public string Description { get; set; }
            public string ObjectId { get; set; }
            public string ContainerName { get; set; }
            public string ContainerId { get; set; }
            public DateTime StartTime { get; set; }
            public DateTime EndTime { get; set; }

            public string Message { get; set; }
            public string Variables { get; set; }
            public string Expressions { get; set; }
            public string Properties {get; set; }

            public EventData()
            {
                EventType = "";
                TaskName = "";
                PackageName = "";
                TaskType = "";
                Description = "";
                ObjectId = "";
                ContainerId = "";
                ContainerName = "";
                Message = "";
                Variables = "";
                Expressions = "";
                Properties = "";
            }
        }

        private const string EXTENSION = ".dtsCSVLog";

        private static Dictionary<string, Dictionary<object, DtsProperty>> _expressionCache = new Dictionary<string, Dictionary<object, DtsProperty>>();
        private static bool _errorFound = false;

        private MemoryStream _memoryStream;
        private StreamWriter _memoryWriter;
        private StreamWriter _writer;
        private bool _packageLoaded = false;
        private bool _headerDone = false;        
        //private bool _verbose = false;
        private object _lockHolder = new object();
        private Guid _executionGuid = Guid.Empty;
        private Uri _packageUri = null;
        private string _executionLabel = string.Empty;

        private PackageInfo _packageInfo;

        public CSVLogProvider()
        {
        }       

        #region ILogProvider Members

        public void Configure(string configurationString)
        {
            if (string.IsNullOrEmpty(configurationString))
            {
                throw new ArgumentNullException("CSVLogProvider needs to be configured through LogProviderArguments command line parameter.");
            }

            string[] configurationValues = configurationString.Split(new char[] { ';' });

            string filePath = GetConfigurationValueFromKey(configurationValues, "FILE");
            if (filePath == string.Empty)
            {
                throw new ArgumentException(@"CSVLogProvider needs to have at least the FILE option specified. eg: FILE=c:\csvlog" + EXTENSION + ".");
            }
            filePath = filePath.Replace("{date}", DateTime.Now.ToString("yyyyMMdd"));
            filePath = filePath.Replace("{time}", DateTime.Now.ToString("HHmmss"));
            filePath = filePath.Replace("{filename}", Path.GetFileNameWithoutExtension(_packageUri.AbsolutePath));
            
            filePath = filePath.Trim();
            if (Path.GetExtension(filePath) != EXTENSION) filePath += EXTENSION;

            Console.WriteLine("CSV Log file will be: {0}", filePath);
            //Console.WriteLine("Verbose: {0}", _verbose.ToString());
            _writer = new StreamWriter(filePath, false, Encoding.UTF8, 512);

            string executionLabel = GetConfigurationValueFromKey(configurationValues, "LABEL");
            if (executionLabel != string.Empty)
            {
                Console.WriteLine("CSV Log will use the Execution Label: {0}", executionLabel);
                _executionLabel = executionLabel;
            }

            //TO BE IMPLEMENTED
            //if (configurationValues.GetUpperBound(0) == 1)
            //{
            //    string verboseOption = configurationValues[1];
            //    if (!verboseOption.StartsWith("VERBOSE="))
            //    {
            //        throw new ArgumentException("CSVLogProvider does not recognize the second option. The only supported option is VERBOSE eg: VERBOSE=1" + EXTENSION + " .");
            //    }
            //}

            // Create memory writer to hold log information that happens before package execution
            // and that doesn't have all the information to fill the header of the file
            _memoryStream = new MemoryStream();
            _memoryWriter = new StreamWriter(_memoryStream);
        }

        public void ShowVersionInformation()
        {
            Assembly executingAssembly = System.Reflection.Assembly.GetExecutingAssembly();
            string copyright = ((AssemblyCopyrightAttribute)executingAssembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), true)[0]).Copyright;
            string version = executingAssembly.GetName().Version.ToString();

            Console.WriteLine("CSVLogProvider " + copyright + " v." + version);
        }

        public void SetExecutionGuid(Guid executionGuid)
        {
            _executionGuid = executionGuid;
        }

        public void SetPackageInfo(Uri packageUri)
        {
            _packageUri = packageUri;
        }

        public void LogStartPackageLoading()
        {
            EventData eventData = new EventData();
            eventData.EventType = "PackageLoadStart";
            eventData.PackageName = "";
            eventData.ObjectId = "";
            eventData.StartTime = DateTime.Now;
            eventData.EndTime = DateTime.Now;
            eventData.Message = "Loading Package...";                                   

            WriteCSVData(eventData);            
        }

        public void LogEndPackageLoading()
        {
            StringBuilder sb = new StringBuilder();
            XmlWriter xw = CrateXmlWriter(sb);
            xw.WriteStartElement("message");
            xw.WriteElementString("result", "Success");
            xw.WriteEndElement();
            xw.Flush(); 
            
            EventData eventData = new EventData();
            eventData.EventType = "PackageLoadEnd";
            eventData.PackageName = "";
            eventData.ObjectId = "";
            eventData.StartTime = DateTime.Now;
            eventData.EndTime = DateTime.Now;
            eventData.Message = sb.ToString();

            WriteCSVData(eventData);

            _packageLoaded = true;
        }

        public void LogEndPackageLoading(Exception exception)
        {
            StringBuilder sb = new StringBuilder();
            XmlWriter xw = CrateXmlWriter(sb);
            xw.WriteStartElement("message");
            xw.WriteElementString("result", "Failure");
            xw.WriteElementString("exception", exception.Message);
            xw.WriteEndElement();
            xw.Flush();
            
            EventData eventData = new EventData();
            eventData.EventType = "PackageLoadEnd";
            eventData.PackageName = "";
            eventData.ObjectId = "";
            eventData.StartTime = DateTime.Now;
            eventData.EndTime = DateTime.Now;
            eventData.Message = sb.ToString();

            WriteCSVData(eventData);                        
        }
                
        public void LogStartPackageExecution(Microsoft.SqlServer.Dts.Runtime.Package package)
        {          
            // Write file header
            WriteCSVHeader(package);

            // Flush logged data gathered during loading to disk
            FlushLoadingLogToDisk();

            // Write event data
            EventData eventData = new EventData();
            eventData.EventType = "PackageStart";
            eventData.PackageName = package.Name;
            eventData.ObjectId = package.ID;
            eventData.StartTime = DateTime.Now;
            eventData.EndTime = DateTime.Now;           

            WriteCSVData(eventData);       
        }

        public void LogEndPackageExecution(Microsoft.SqlServer.Dts.Runtime.Package package, int warningCount, int errorCount, Microsoft.SqlServer.Dts.Runtime.DTSExecResult result)
        {
            StringBuilder sb = new StringBuilder();
            XmlWriter xw = CrateXmlWriter(sb);
            xw.WriteStartElement("message");
            xw.WriteElementString("result", result.ToString());
            xw.WriteElementString("warnings", warningCount.ToString());
            xw.WriteElementString("errors", errorCount.ToString());
            xw.WriteEndElement();
            xw.Flush();

            EventData eventData = new EventData();
            eventData.EventType = "PackageEnd";
            eventData.PackageName = package.Name;
            eventData.ObjectId = package.ID;
            eventData.StartTime = DateTime.Now;
            eventData.EndTime = DateTime.Now;            
            eventData.Message = sb.ToString();

            WriteCSVData(eventData);
            UpdateCSVHeader();
        }

        public void LogEndPackageExecution(Microsoft.SqlServer.Dts.Runtime.Package package, int warningCount, int errorCount, Microsoft.SqlServer.Dts.Runtime.DTSExecResult result, Microsoft.SqlServer.Dts.Runtime.DtsException exception)
        {
            StringBuilder sb = new StringBuilder();
            XmlWriter xw = CrateXmlWriter(sb);
            xw.WriteStartElement("message");
            xw.WriteElementString("result", result.ToString());
            xw.WriteElementString("warnings", warningCount.ToString());
            xw.WriteElementString("errors", errorCount.ToString());
            xw.WriteElementString("exception", exception.Message);
            xw.WriteEndElement();
            xw.Flush();

            EventData eventData = new EventData();
            eventData.EventType = "PackageEnd";
            eventData.PackageName = package.Name;
            eventData.ObjectId = package.ID;
            eventData.StartTime = DateTime.Now;
            eventData.EndTime = DateTime.Now;
            eventData.Message = sb.ToString();

            WriteCSVData(eventData);
            UpdateCSVHeader();
        }

        public void LogEventData(string eventName, Microsoft.SqlServer.Dts.Runtime.DtsObject dtsobj, string message)
        {
            StringBuilder sbVariables = new StringBuilder();
            StringBuilder sbProperties = new StringBuilder();
            StringBuilder sbExpressions = new StringBuilder();

            Dictionary<object, DtsProperty> propertiesExpressions = new Dictionary<object, DtsProperty>();

            lock (_lockHolder)
            {
                EventData eventData = new EventData();

                if (dtsobj is EventsProvider)
                {
                    EventsProvider ep = dtsobj as EventsProvider;
                    EventsProvider parentEp = ep.Parent != null ? (EventsProvider)ep.Parent : ep;
                    Package pkg = null;
                    TaskHost th = dtsobj as TaskHost;

                    // Find package 
                    EventsProvider dummyEp = ep.Parent as EventsProvider;
                    if (dummyEp == null)
                    {
                        if (ep is Package)
                        {
                            pkg = ep as Package;
                        }
                    }
                    else
                    {
                        while (dummyEp != null)
                        {
                            if (dummyEp is Package)
                            {
                                pkg = dummyEp as Package;
                                dummyEp = null;
                            }
                            else
                            {
                                dummyEp = dummyEp.Parent as EventsProvider;
                            }
                        }
                    }

                    eventData.EventType = eventName;
                    eventData.TaskName = ep.Name;


                    // If the event has been raised from a package (a not from a task inside)
                    // then find all expression used in that package and store it for later use
                    if (pkg != null)
                    {
                         // If package is beign loaded the ID is null                      
                        if (pkg.ID != null)
                        {
                            string cacheKey = pkg.ID.ToString();

                            if (CSVLogProvider._expressionCache.ContainsKey(cacheKey))
                            {
                                propertiesExpressions = CSVLogProvider._expressionCache[cacheKey];
                            }
                            else
                            {
                                propertiesExpressions = GetExpressions(pkg);
                                CSVLogProvider._expressionCache[cacheKey] = propertiesExpressions;
                            }

                            eventData.ObjectId = ep.ID.ToString();
                            eventData.ContainerId = parentEp.ID.ToString();
                        }

                        eventData.PackageName = pkg.Name;
                    }

                    eventData.Description = ep.Description;                    
                    eventData.ContainerName = parentEp.Name;                    
                    eventData.StartTime = ep.StartTime;
                    eventData.EndTime = ep.StopTime;
                    eventData.Message = message;
                    

                    // Store variables values 
                    XmlWriter xwVariables = CrateXmlWriter(sbVariables);
                    xwVariables.WriteStartElement("variables");
                    foreach (Variable v in ep.Variables)
                    {
                        if (v.Namespace != "System")
                        {
                            xwVariables.WriteStartElement("variable");

                            string name = v.GetPackagePath();
                            string value = v.Value.ToString();
                            string expression = "";

                            if (v.EvaluateAsExpression)
                            {
                                expression = v.Expression;
                            }

                            if (!string.IsNullOrEmpty(name))
                            {
                                xwVariables.WriteAttributeString("name", name);
                                xwVariables.WriteAttributeString("value", value);
                                if (!string.IsNullOrEmpty(expression)) xwVariables.WriteAttributeString("expression", expression);
                            }

                            xwVariables.WriteEndElement();
                        }                        
                    }
                    xwVariables.WriteEndElement();
                    xwVariables.Flush();
                    eventData.Variables = sbVariables.ToString(); 

                    // Show expression values                    
                    XmlWriter xwExpressions = CrateXmlWriter(sbExpressions);
                    xwExpressions.WriteStartElement("expressions");

                    foreach (KeyValuePair<object, DtsProperty> kvp in propertiesExpressions)
                    {
                        xwExpressions.WriteStartElement("expression");

                        IDTSPropertiesProvider dpp = kvp.Key as IDTSPropertiesProvider;
                        DtsProperty p = kvp.Value;

                        string value = GetPropertyValue(dpp, p);

                        xwExpressions.WriteAttributeString("name", p.GetPackagePath(dpp));
                        xwExpressions.WriteAttributeString("value", value);
                        
                        xwExpressions.WriteEndElement();
                    }
                    xwExpressions.WriteEndElement();
                    xwExpressions.Flush();
                    eventData.Expressions = sbExpressions.ToString();

                    // On error store all task properties values
                    if ((eventName == "OnError") && (th != null) && (_errorFound == false))
                    {
                        _errorFound = true;

                        XmlWriter xwProperties = CrateXmlWriter(sbProperties);
                        xwProperties.WriteStartElement("task");

                        foreach (DtsProperty p in th.Properties)
                        {
                            string found_value = string.Empty;
                            object value = null;
                            if (p.Get)
                            {
                                found_value = GetPropertyValue(th, p);
                            }
                            else
                            {
                                found_value = "*** WRITE ONLY ***";
                            }
                            xwProperties.WriteStartElement("property");
                            xwProperties.WriteAttributeString("path", p.GetPackagePath(th));
                            xwProperties.WriteAttributeString("value", found_value);
                            xwProperties.WriteEndElement();

                            if ((p.Name == "Connection") && (value != null))
                            {
                                ConnectionManager cm = pkg.Connections[value.ToString()];
                                if (cm != null)
                                {
                                    xwProperties.WriteStartElement("connection");

                                    foreach (DtsProperty cmp in cm.Properties)
                                    {
                                        string cm_found_value = string.Empty;
                                        if (cmp.Get)
                                        {
                                            cm_found_value = GetPropertyValue(cm, cmp);
                                        }
                                        else
                                        {
                                            cm_found_value = "*** WRITE ONLY ***";
                                        }
                                        xwProperties.WriteStartElement("property");
                                        xwProperties.WriteAttributeString("path", cmp.GetPackagePath(cm));
                                        xwProperties.WriteAttributeString("value", cm_found_value);
                                        xwProperties.WriteEndElement();
                                    }

                                    xwProperties.WriteEndElement();
                                }
                            }
                        }

                        xwProperties.WriteEndElement();
                        xwProperties.Flush();

                        eventData.Properties = sbProperties.ToString();
                    }
                    else
                    {
                        eventData.EventType = eventName;
                        eventData.Message = message;
                    }

                    // If the event has been generated during package loading,
                    // try to store as much as info as possible, so that, in case of load failure, something helpful
                    // can be written to the log anyway
                    if (!_packageLoaded && _packageInfo == null)
                    {
                        _packageInfo = new PackageInfo();
                        _packageInfo.Name = pkg.Name;
                        _packageInfo.Version = string.Format("{0}.{1}.{2}", pkg.VersionMajor, pkg.VersionMinor, pkg.VersionBuild);
                        _packageInfo.PackageGUID = string.Format("{0}", pkg.ID);
                        _packageInfo.VersionGUID = pkg.VersionGUID.ToString();
                        _packageInfo.ExecutionGUID = _executionGuid.ToString().ToUpperInvariant();
                        _packageInfo.UserName = Environment.UserName;
                        _packageInfo.MachineName = Environment.MachineName;
                        _packageInfo.ExecutionDateTime = DateTime.Now.ToString("s");
                        _packageInfo.ExecutionLabel = _executionLabel;
                    }
                    
                    WriteCSVData(eventData);
                }
            }
        }

        #endregion
    
        #region IDisposable Members

        public void Dispose()
        {
            if (!_headerDone && !_packageLoaded && _packageInfo != null)
            {
                WriteCSVHeader();
                FlushLoadingLogToDisk();
            }

            _memoryWriter.Close();
 	        _writer.Close();
        }

        #endregion

        private string GetPropertyValue(IDTSPropertiesProvider dpp, DtsProperty p)
        {
            string found_value = string.Empty;

            object value = p.GetValue(dpp);
            if (value != null) found_value = value.ToString();
            else found_value = string.Empty;

            return found_value;
        }

        private XmlWriter CrateXmlWriter(StringBuilder targetStringBuilder)
        {
            targetStringBuilder.Length = 0;

            XmlWriterSettings xwSettings = new XmlWriterSettings();                  
            xwSettings.Encoding = Encoding.UTF8;
            xwSettings.OmitXmlDeclaration = true;

            XmlWriter writer = XmlWriter.Create(targetStringBuilder, xwSettings);            
            return writer;
        }

        private void WriteCSVHeader()
        {
            if (!_headerDone)
            {
                _writer.WriteLine("#__DataFileVersion: 4");
                _writer.WriteLine("#FileStatus: OPEN ");
                _writer.WriteLine("#Package: " + _packageInfo.Name);
                _writer.WriteLine("#Version: {0}", _packageInfo.Version);
                _writer.WriteLine("#PackageGUID: {0}", _packageInfo.PackageGUID);
                _writer.WriteLine("#VersionGUID: {0}", _packageInfo.VersionGUID);
                _writer.WriteLine("#ExecutionGUID: {{{0}}}", _packageInfo.ExecutionGUID);
                _writer.WriteLine("#UserName: {0}", _packageInfo.UserName);
                _writer.WriteLine("#MachineName: {0}", _packageInfo.MachineName);
                _writer.WriteLine("#ExecutionDateTime: {0}", _packageInfo.ExecutionDateTime);
                _writer.WriteLine("#ExecutionLabel: {0}", _packageInfo.ExecutionLabel);
                _writer.WriteLine("EventType;;TaskName;;PackageName;;TaskType;;Description;;ObjectId;;ContainerName;;ContainerId;;StartTime;;EndTime;;Message;;Variables;;Expressions;;Properties");

                _headerDone = true;
            }
        }

        private void WriteCSVHeader(Package package)
        {
            if (!_headerDone)
            {
                _writer.WriteLine("#__DataFileVersion: 4");
                _writer.WriteLine("#FileStatus: OPEN ");
                _writer.WriteLine("#Package: " + package.Name);
                _writer.WriteLine("#Version: {0}.{1}.{2}", package.VersionMajor, package.VersionMinor, package.VersionBuild);
                _writer.WriteLine("#PackageGUID: {0}", package.ID);
                _writer.WriteLine("#VersionGUID: {0}", package.VersionGUID);
                _writer.WriteLine("#ExecutionGUID: {{{0}}}", _executionGuid.ToString().ToUpperInvariant());
                _writer.WriteLine("#UserName: {0}", Environment.UserName);
                _writer.WriteLine("#MachineName: {0}", Environment.MachineName);
                _writer.WriteLine("#ExecutionDateTime: {0}", DateTime.Now.ToString("s"));
                _writer.WriteLine("#ExecutionLabel: {0}", _executionLabel);
                _writer.WriteLine("EventType;;TaskName;;PackageName;;TaskType;;Description;;ObjectId;;ContainerName;;ContainerId;;StartTime;;EndTime;;Message;;Variables;;Expressions;;Properties");

                _headerDone = true;
            }            
        }


        private void UpdateCSVHeader()
        {
            // Set position to header data
            _writer.BaseStream.Seek(39, SeekOrigin.Begin);
            _writer.BaseStream.Write(Encoding.UTF8.GetBytes("CLOSE"),0,5);

            // Set position to the end of file
            _writer.BaseStream.Seek(0, SeekOrigin.End);
        }

        private void FlushLoadingLogToDisk()
        {          
            _memoryWriter.Flush();
            if (_memoryStream.Length != 0)
            {
                _memoryStream.Position = 0;
                StreamReader sr = new StreamReader(_memoryStream);
                _writer.Write(sr.ReadToEnd());
                sr.Close();
            }
        }

        //private void WriteDataToMemory(EventData eventData)
        //{
        //    _memoryWriter.WriteLine("{0};;{1};;{2};;{3};;{4};;{5};;{6};;{7};;{8};;{9};;{10};;{11};;{12};;{13}",
        //                       eventData.EventType,
        //                       eventData.TaskName,
        //                       eventData.PackageName,
        //                       eventData.TaskType,
        //                       eventData.Description,
        //                       eventData.ObjectId,
        //                       eventData.ContainerName,
        //                       eventData.ContainerId,
        //                       eventData.StartTime.ToString("s"),
        //                       eventData.EndTime.ToString("s"),
        //                       eventData.Message,
        //                       eventData.Variables,
        //                       eventData.Expressions,
        //                       eventData.Properties);
        //}

        private void WriteCSVData(EventData eventData)
        {
            StreamWriter sw = null;

            if (_packageLoaded)
                sw = _writer;
            else
                sw = _memoryWriter;

            sw.WriteLine("{0};;{1};;{2};;{3};;{4};;{5};;{6};;{7};;{8};;{9};;{10};;{11};;{12};;{13}", 
                    eventData.EventType, 
                    eventData.TaskName, 
                    eventData.PackageName, 
                    eventData.TaskType, 
                    eventData.Description, 
                    eventData.ObjectId,
                    eventData.ContainerName, 
                    eventData.ContainerId, 
                    eventData.StartTime.ToString("s"), 
                    eventData.EndTime.ToString("s"), 
                    eventData.Message, 
                    eventData.Variables, 
                    eventData.Expressions,
                    eventData.Properties);

            Console.WriteLine("[CSV Logger] {0} {3} [{1}\\{2}] {4}",
                    eventData.EndTime.ToString("s"),
                    eventData.ContainerName,
                    eventData.TaskName,
                    eventData.EventType,
                    eventData.Message.Replace("\r\n", " ")
                    );           
        }        

        private Dictionary<object, DtsProperty> GetExpressions(Package package)
        {
            // Find all properties
            Dictionary<object, DtsProperty> propDict = new Dictionary<object, DtsProperty>();

            Connections connections = package.Connections;
            foreach (ConnectionManager connMan in connections)
            {
                Dictionary<object, DtsProperty> dummyDict = GetPropertiesWithExpressions(connMan);

                foreach (KeyValuePair<object, DtsProperty> kvp in dummyDict)
                {
                    propDict.Add(kvp.Key, kvp.Value);
                }
            }

            Executables executables = package.Executables;
            foreach (Executable exec in executables)
            {
                if (exec is IDTSPropertiesProvider)
                {
                    Dictionary<object, DtsProperty> dummyDict = GetPropertiesWithExpressions(exec as IDTSPropertiesProvider);

                    foreach (KeyValuePair<object, DtsProperty> kvp in dummyDict)
                    {
                        propDict.Add(kvp.Key, kvp.Value);
                    }
                }
            }

            foreach (KeyValuePair<object, DtsProperty> kvp in propDict)
            {
                object dpp = kvp.Key;
                DtsProperty p = kvp.Value;

                object value = p.GetValue(dpp);
            }

            return propDict;
        }

        private Dictionary<object, DtsProperty> GetPropertiesWithExpressions(IDTSPropertiesProvider dpp)
        {
            Dictionary<object, DtsProperty> propDict = new Dictionary<object, DtsProperty>();

            if (dpp != null)
            {
                foreach (DtsProperty p in dpp.Properties)
                {
                    object expression = p.GetExpression(dpp);
                    if (expression != null)
                    {
                        if (!propDict.ContainsKey(dpp)) propDict.Add(dpp, p);
                    }
                }
            }

            return propDict;
        }

        private string GetConfigurationValueFromKey(string[] configurationValues, string key)
        {
            string token = key.Trim() + "=";
            string result = string.Empty;

            foreach (string s in configurationValues)
            {
                if (s.StartsWith(token))
                {
                    result = s.Replace(token, string.Empty); ;

                    break;
                }
            }

            return result;
        }
    }
}
