using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Reflection;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;

namespace DTLoggedExec.LogProviders
{
    public class ConsoleLogProvider : ILogProvider
    {
        private static Dictionary<string, Dictionary<object, DtsProperty>> _expressionCache = new Dictionary<string, Dictionary<object, DtsProperty>>();
        private static bool _errorFound = false;

        private object _lockHolder = new object();

        #region ILogProvider Members

        public void Configure(string configurationString)
        {
            // No configuration needed
            return;
        }

        public void SetExecutionGuid(Guid executionGuid)
        { 
            // No need to store execution GUID here
            return;
        }

        public void SetPackageInfo(Uri packageUri)
        {
            // No need to store Package info
            return;
        }

        public void ShowVersionInformation()
        {
            Assembly executingAssembly = System.Reflection.Assembly.GetExecutingAssembly();
            string copyright = ((AssemblyCopyrightAttribute)executingAssembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), true)[0]).Copyright;
            string version = executingAssembly.GetName().Version.ToString();

            Console.WriteLine("ConsoleLogProvider " + copyright + " v." + version);
        }      
        
        public void LogEventData(string eventName, Microsoft.SqlServer.Dts.Runtime.DtsObject dtsobj, string message)
        {           
            Dictionary<object, DtsProperty> propertiesExpressions = new Dictionary<object, DtsProperty>();

            lock (_lockHolder)
            {                
                if (dtsobj is EventsProvider)
                {
                    EventsProvider ep = dtsobj as EventsProvider;
                    EventsProvider parentEp = ep.Parent != null ? (EventsProvider)ep.Parent : ep;
                    Package pkg = null;
                    TaskHost th = dtsobj as TaskHost;                                    

                    // Find package 
                    EventsProvider dummyEp = ep.Parent as EventsProvider;
                    if (dummyEp == null)
                    {
                        if (ep is Package)
                        {
                            pkg = ep as Package;
                        }
                    }
                    else
                    {
                        while (dummyEp != null)
                        {
                            if (dummyEp is Package)
                            {
                                pkg = dummyEp as Package;
                                dummyEp = null;
                            }
                            else
                            {
                                dummyEp = dummyEp.Parent as EventsProvider;
                            }
                        }
                    }                    

                    Console.WriteLine(eventName + ": " + ep.Name);


                    // If the event has been raised from a package (a not from a task inside)
                    // then find all expression used in that package and store it for later use
                    if (pkg != null)
                    {
                        // If package is beign loaded the ID is null                      
                        if (pkg.ID != null)
                        {
                            string cacheKey = pkg.ID.ToString();

                            if (ConsoleLogProvider._expressionCache.ContainsKey(cacheKey))
                            {
                                propertiesExpressions = ConsoleLogProvider._expressionCache[cacheKey];
                            }
                            else
                            {
                                propertiesExpressions = GetExpressions(pkg);
                                ConsoleLogProvider._expressionCache[cacheKey] = propertiesExpressions;
                            }                            
                        }

                        Console.WriteLine("\tPackage Name:\t\t" + pkg.Name);
                    }
                    
                    if (th != null)
                    {
                        Console.WriteLine("\tTask Type:\t\t" + th.InnerObject.GetType().Name + " (" + ep.CreationName + ")");
                    }
                    else
                    {
                        Console.WriteLine("\tObject Type:\t\t" + " (" + ep.CreationName + ")");
                    }
                    
                    Console.WriteLine("\tDescription:\t\t" + ep.Description);
                    Console.WriteLine("\tObject ID:\t\t{0}", ep.ID);
                    Console.WriteLine("\tContainer Name:\t\t" + parentEp.Name);
                    Console.WriteLine("\tContainer ID:\t\t{0}", parentEp.ID);
                    
                    //  Sometimes the engine reports that an event Started AFTER it's finished! :-|
                    if (ep.StartTime > ep.StopTime)
                    {
                        Console.WriteLine("\tStartTime:\t\t" + ep.StopTime);
                        Console.WriteLine("\tEndTime:\t\t" + ep.StartTime);
                    }
                    else
                    {
                        Console.WriteLine("\tStartTime:\t\t" + ep.StartTime);
                        Console.WriteLine("\tEndTime:\t\t" + ep.StopTime);
                    }
                    Console.WriteLine("\tMessage:\t\t" + message);

                    // Print variables values
                    Console.WriteLine("\tVariables:");
                    foreach (Variable v in ep.Variables)
                    {                                                
                        if (v.Namespace != "System")
                        {
                            Console.Write("\t\t" + v.GetPackagePath() + " = " + v.Value.ToString());

                            if (v.EvaluateAsExpression)
                            {
                                Console.WriteLine(" ( Evaluated from expression: " + v.Expression + " )");
                            }
                            else
                            {
                                Console.WriteLine();
                            }
                        }                        
                    }

                    // Show expression values
                    Console.WriteLine("\tExpressions:");
                    foreach (KeyValuePair<object, DtsProperty> kvp in propertiesExpressions)
                    {
                        object dpp = kvp.Key;
                        DtsProperty p = kvp.Value;                        

                        object value = p.GetValue(dpp);
                        Console.WriteLine("\t\t" + p.GetPackagePath(dpp) + ": " + value.ToString());
                    }

                    // On error show all task properties values
                    if ((eventName == "OnError") && (th != null) && (_errorFound == false))
                    {
                        _errorFound = true;

                        Console.WriteLine("\tProperties:");

                        foreach (DtsProperty p in th.Properties)
                        {
                            string found_value = string.Empty;
                            object value = null;
                            if (p.Get)
                            {
                                value = p.GetValue(th);
                                if (value != null) found_value = value.ToString();
                                else found_value = string.Empty;
                            }
                            else
                            {
                                found_value = "*** WRITE ONLY ***";
                            }
                            Console.WriteLine("\t\t" + p.GetPackagePath(th) + ": " + found_value);

                            if ((p.Name == "Connection") && (value != null))
                            {
                                ConnectionManager cm = pkg.Connections[value.ToString()];
                                if (cm != null)
                                {
                                    foreach (DtsProperty cmp in cm.Properties)
                                    {
                                        string cm_found_value = string.Empty; 
                                        if (cmp.Get)
                                        {
                                            object cmvalue = cmp.GetValue(cm);
                                            if (cmvalue != null) cm_found_value = cmvalue.ToString();
                                            else cm_found_value = string.Empty;
                                        }
                                        else
                                        {
                                            cm_found_value = "*** WRITE ONLY ***";
                                        }
                                        Console.WriteLine("\t\t\t" + cmp.GetPackagePath(cm) + ": " + cm_found_value);
                                    }
                                }
                            }
                        }
                        
                        // If the error happened in the Data Flow try to show where the error happened
                        // and the row that generated the error
                        //if ((th.InnerObject is MainPipe))
                        //{
                        //    Console.WriteLine("\tDataFlow objects");
                        //    MainPipe mp = th.InnerObject as MainPipe;

                        //    IDTSComponentMetaDataCollection90 dataFlowObjects = mp.ComponentMetaDataCollection;
                        //    foreach (IDTSComponentMetaData90 dfObj in dataFlowObjects)
                        //    {
                        //        Console.WriteLine("\t\t" + dfObj.Name);

                        //    }
                        //}                   

                    }
                }
                else
                {
                    Console.WriteLine(eventName + ": " + message);
                }                
            }
        }

        public void LogStartPackageLoading()
        {
            Console.WriteLine("Loading Package...");
        }

        public void LogEndPackageLoading()
        {
            Console.WriteLine("Package Loaded.");
        }

        public void LogEndPackageLoading(Exception exception)
        {
            Console.WriteLine();
            Console.WriteLine(exception.Message);
            Console.WriteLine("Error while loading Package.");
        }
        
        public void LogStartPackageExecution(Microsoft.SqlServer.Dts.Runtime.Package package)
        {
            Console.WriteLine(); 
            Console.WriteLine("Package start.");
        }

        public void LogEndPackageExecution(Microsoft.SqlServer.Dts.Runtime.Package package, int warningCount, int errorCount, Microsoft.SqlServer.Dts.Runtime.DTSExecResult result)
        {
            Console.WriteLine();
            Console.WriteLine("Package end.");
        }

        public void LogEndPackageExecution(Microsoft.SqlServer.Dts.Runtime.Package package, int warningCount, int errorCount, Microsoft.SqlServer.Dts.Runtime.DTSExecResult result, Microsoft.SqlServer.Dts.Runtime.DtsException exception)
        {
            Console.WriteLine();
            Console.WriteLine("Package end: " + exception.Message);
        }

        #endregion

        private Dictionary<object, DtsProperty> GetExpressions(Package package)
        {
            // Find all properties
            Dictionary<object, DtsProperty> propDict = new Dictionary<object, DtsProperty>();

            Connections connections = package.Connections;
            foreach (ConnectionManager connMan in connections)
            {
                Dictionary<object, DtsProperty> dummyDict = GetPropertiesWithExpressions(connMan);

                foreach (KeyValuePair<object, DtsProperty> kvp in dummyDict)
                {
                    propDict.Add(kvp.Key, kvp.Value);
                }
            }

            Executables executables = package.Executables;
            foreach (Executable exec in executables)
            {
                if (exec is IDTSPropertiesProvider)
                {
                    Dictionary<object, DtsProperty> dummyDict = GetPropertiesWithExpressions(exec as IDTSPropertiesProvider);

                    foreach (KeyValuePair<object, DtsProperty> kvp in dummyDict)
                    {
                        propDict.Add(kvp.Key, kvp.Value);
                    }
                }
            }

            foreach (KeyValuePair<object, DtsProperty> kvp in propDict)
            {
                object dpp = kvp.Key;
                DtsProperty p = kvp.Value;

                object value = p.GetValue(dpp);
                //Console.WriteLine("\t" + p.GetPackagePath(dpp) + ": " + value.ToString());
            }

            return propDict;
        }

        private Dictionary<object, DtsProperty> GetPropertiesWithExpressions(IDTSPropertiesProvider dpp)
        {
            Dictionary<object, DtsProperty> propDict = new Dictionary<object, DtsProperty>();

            if (dpp != null)
            {
                foreach (DtsProperty p in dpp.Properties)
                {
                    object expression = p.GetExpression(dpp);
                    if (expression != null)
                    {
                        //object value = p.GetValue(dpp);
                        //Console.WriteLine("\t" + p.GetPackagePath(dpp) + ": " + value.ToString());
                        if (!propDict.ContainsKey(dpp)) propDict.Add(dpp, p);
                    }
                }
            }

            return propDict;
        }

        #region IDisposable Members

        public void Dispose()
        {
            return;
        }

        #endregion
    }
}
