using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using DTLoggedExec.LogProviders;

namespace DTLoggedExec
{
    [Flags]
    public enum PackageEvents
    {
        None = 0,       
        Progress = 4,
        Custom = 8,
        Information = 16,
        ExecutionStatusChanged = 32,
        TaskFailed = 64,
        PreValidate = 128,
        PostValidate = 256,
        PreExecute = 512,
        PostExecute = 1024,
        Warning = 2048,
        Error = 4096
    }

    public class CustomEventHandler : DefaultEvents, IDisposable
    {
        private delegate void DataFlowProfileProviderDelegate(TaskHost taskHost, string eventName, string eventText, object[] arguments, string subComponent);

        private DataFlowProfileProviderDelegate _dataFlowProfileHandler = null;
        private DataFlowProfileProvider _dfpp = null;

        private PackageEvents _selectedEvents;
        private int _errorsCount = 0;
        private int _warningsCount = 0;
        private bool _profileDataFlow = false;

        private ILogProvider _logProvider;

        public int ErrorsCount
        {
            get { return _errorsCount; }
        }

        public int WarningsCount
        {
            get { return _warningsCount; }
        }

        public CustomEventHandler(ILogProvider logProvider, PackageEvents selectedEvents)
            : this(logProvider, selectedEvents, null)
        {
        }

        public CustomEventHandler(ILogProvider logProvider, PackageEvents selectedEvents, DataFlowProfileProvider dataFlowProfileProvider)
        {
            _logProvider = logProvider;
            _selectedEvents = selectedEvents;

            AttachDataFlowProfiler(dataFlowProfileProvider);
        }

        public void AttachDataFlowProfiler(DataFlowProfileProvider dataFlowProfileProvider)
        {
            _profileDataFlow = (dataFlowProfileProvider != null);

            if (_profileDataFlow)
            {
                _dfpp = dataFlowProfileProvider;
                _dataFlowProfileHandler = new DataFlowProfileProviderDelegate(dataFlowProfileProvider.ProcessDataFlowData);
            }
        }

        public void FireLoadPackageStart()
        {
            _logProvider.LogStartPackageLoading();
        }

        public void FireLoadPackageEnd()
        {
            _logProvider.LogEndPackageLoading();
        }

        public void FireLoadPackageEnd(Exception exception)
        {
            _logProvider.LogEndPackageLoading(exception);
            if (_dfpp != null) _dfpp.Dispose();
        }

        public void FirePackageStart(Package package)
        {
            _logProvider.LogStartPackageExecution(package);
        }

        public void FirePackageEnd(Package package, DTSExecResult result)
        {
            _logProvider.LogEndPackageExecution(package, _warningsCount, _errorsCount, result);
            if (_dfpp != null) _dfpp.Dispose();
        }

        public void FirePackageEnd(Package package, DTSExecResult result, DtsException exception)
        {
            _logProvider.LogEndPackageExecution(package, _warningsCount, _errorsCount, result, exception);
            if (_dfpp != null) _dfpp.Dispose();
        }

        public override void OnProgress(TaskHost taskHost, string progressDescription, int percentComplete, int progressCountLow, int progressCountHigh, string subComponent, ref bool fireAgain)
        {
            if ((_selectedEvents & PackageEvents.Progress) == PackageEvents.Progress)
            {
                fireAgain = true;
                LogData("OnProgress", taskHost, progressDescription);
            }
            else
            {
                fireAgain = false;
            }
        }

        public override void OnCustomEvent(TaskHost taskHost, string eventName, string eventText, ref object[] arguments, string subComponent, ref bool fireAgain)
        {
            // Here the custom event "OnPipelineRowSent" should be processed so that
            // we can keep track of how many rows are processed in the DataFlow            
            if (_profileDataFlow) _dataFlowProfileHandler(taskHost, eventName, eventText, arguments, subComponent);

            if ((_selectedEvents & PackageEvents.Custom) == PackageEvents.Custom)
            {
                fireAgain = true;
                LogData("OnCustomEvent::" + eventName, taskHost, eventText);
            }
            else
            {
                fireAgain = _profileDataFlow;
            }
        }

        public override void OnInformation(DtsObject source, int informationCode, string subComponent, string description, string helpFile, int helpContext, string idofInterfaceWithError, ref bool fireAgain)
        {
            if ((_selectedEvents & PackageEvents.Information) == PackageEvents.Information)
            {
                fireAgain = true;
                LogData("OnInformation", source, description);
            }
            else
            {
                fireAgain = false;
            }
        }

        public override void OnExecutionStatusChanged(Executable exec, DTSExecStatus newStatus, ref bool fireAgain)
        {
            if ((_selectedEvents & PackageEvents.ExecutionStatusChanged) == PackageEvents.ExecutionStatusChanged)
            {
                fireAgain = true;
                LogData("OnExecutionStatusChanged", exec);
            }
            else
            {
                fireAgain = false;
            }
        }

        public override void OnTaskFailed(TaskHost taskHost)
        {
            if ((_selectedEvents & PackageEvents.TaskFailed) == PackageEvents.TaskFailed)
            {               
                LogData("OnTaskFailed", taskHost);
            }
        }

        public override void OnPreValidate(Executable exec, ref bool fireAgain)
        {
            if ((_selectedEvents & PackageEvents.PreValidate) == PackageEvents.PreValidate)
            {
                fireAgain = true;
                LogData("OnPreValidate", exec);
            }
            else
            {
                fireAgain = false;
            }
        }

        public override void OnPreExecute(Executable exec, ref bool fireAgain)
        {
            if (_profileDataFlow)
                _dataFlowProfileHandler(exec as TaskHost, "OnPreExecute", "", null, "");

            if ((_selectedEvents & PackageEvents.PreExecute) == PackageEvents.PreExecute)
            {
                fireAgain = true;
                LogData("OnPreExecute", exec);
            }
            else
            {
                fireAgain = _profileDataFlow;
            }
        }

        public override void OnPostValidate(Executable exec, ref bool fireAgain)
        {
            if ((_selectedEvents & PackageEvents.PostValidate) == PackageEvents.PostValidate)
            {
                fireAgain = true;
                LogData("OnPostValidate", exec);
            }
            else
            {
                fireAgain = false;
            }
        }

        public override void OnPostExecute(Executable exec, ref bool fireAgain)
        {
            if (_profileDataFlow)
                _dataFlowProfileHandler(exec as TaskHost, "OnPostExecute", "", null, "");

            if ((_selectedEvents & PackageEvents.PostExecute) == PackageEvents.PostExecute)
            {
                fireAgain = true;
                LogData("OnPostExecute", exec);
            }
            else
            {
                fireAgain = _profileDataFlow;
            }
        }

        public override void OnWarning(DtsObject source, int warningCode, string subComponent, string description, string helpFile, int helpContext, string idofInterfaceWithError)
        {
            _warningsCount++;
            if ((_selectedEvents & PackageEvents.Warning) == PackageEvents.Warning)
            {
                LogData("OnWarning", source, description);
            }
        }

        public override bool OnError(DtsObject source, int errorCode, string subComponent, string description, string helpFile, int helpContext, string idofInterfaceWithError)
        {
            _errorsCount++;
            if ((_selectedEvents & PackageEvents.Error) == PackageEvents.Error)
            {
                LogData("OnError", source, description);
            }
            return false;
        }

        private void LogData(string eventName, DtsObject dtsobj)
        {
            _logProvider.LogEventData(eventName, dtsobj, string.Empty);
        }

        private void LogData(string eventName, DtsObject dtsobj, string message)
        {
            _logProvider.LogEventData(eventName, dtsobj, message);
        }

        #region IDisposable Members

        public void Dispose()
        {
            _logProvider.Dispose();
        }

        #endregion
    }
}

